@@ -34,7 +34,7 @@ static AtkStateSet*          gail_button_ref_state_set    (AtkObject       *obj)
 static void                  gail_button_notify_label_gtk (GObject         *obj,
                                                            GParamSpec      *pspec,
                                                            gpointer        data);
-static gint                  gail_button_label_map_gtk    (GtkWidget       *widget,
+static void                  gail_button_label_map_gtk    (GtkWidget       *widget,
                                                            gpointer        data);
 
 static void                  gail_button_real_initialize  (AtkObject       *obj,
@@ -295,6 +295,8 @@ gail_button_real_initialize (AtkObject *obj,
 {
   GailButton *button = GAIL_BUTTON (obj);
   GtkWidget  *label;
+  GtkWidget  *widget;
+  GtkWidget  *child;
 
   ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
 
@@ -318,7 +320,8 @@ gail_button_real_initialize (AtkObject *obj,
                     NULL);
 
 
-  label = get_label_from_button (GTK_WIDGET (data), 0, FALSE);
+  widget = GTK_WIDGET (data);
+  label = get_label_from_button (widget, 0, FALSE);
   if (GTK_IS_LABEL (label))
     {
       if (GTK_WIDGET_MAPPED (label))
@@ -329,11 +332,18 @@ gail_button_real_initialize (AtkObject *obj,
                           G_CALLBACK (gail_button_label_map_gtk),
                           button);
     }
+  child = GTK_BIN (widget)->child;
+  if (GTK_IS_ARROW (child) &&
+      GTK_ARROW (child)->arrow_type == GTK_ARROW_DOWN)
+    button->is_down_arrow = TRUE;
+  else
+    button->is_down_arrow = FALSE;
+    
   set_role_for_button (obj, data);
 
 }
 
-static gint
+static void
 gail_button_label_map_gtk (GtkWidget *widget,
                            gpointer data)
 {
@@ -497,6 +507,13 @@ idle_do_action (gpointer data)
   while (!g_queue_is_empty (gail_button->action_queue)) 
     {
       gint action_number = (gint) g_queue_pop_head (gail_button->action_queue);
+      if (gail_button->is_down_arrow)
+        {
+          if (action_number == 0)
+            action_number = 1;
+          else if (action_number == 1)
+            action_number = 0;
+        }
       switch (action_number)
 	{
 	case 0:
@@ -550,6 +567,13 @@ gail_button_get_description (AtkAction *action,
 
   button = GAIL_BUTTON (action);
 
+  if (button->is_down_arrow)
+    {
+      if (i == 0)
+        i = 1;
+      else if (i == 1)
+        i = 0;
+    }
   switch (i)
     {
     case 0:
@@ -575,6 +599,14 @@ gail_button_get_keybinding (AtkAction *action,
   GailButton *button;
   gchar *return_value = NULL;
 
+  button = GAIL_BUTTON (action);
+  if (button->is_down_arrow)
+    {
+      if (i == 0)
+        i = 1;
+      else if (i == 1)
+        i = 0;
+    }
   switch (i)
     {
     case 0:
@@ -586,7 +618,6 @@ gail_button_get_keybinding (AtkAction *action,
         GtkWidget *label;
         guint key_val; 
 
-        button = GAIL_BUTTON (action);
         widget = GTK_ACCESSIBLE (button)->widget;
         if (widget == NULL)
           /*
@@ -650,7 +681,17 @@ gail_button_action_get_name (AtkAction *action,
                              gint      i)
 {
   G_CONST_RETURN gchar *return_value;
+  GailButton *button;
 
+  button = GAIL_BUTTON (action);
+
+  if (button->is_down_arrow)
+    {
+      if (i == 0)
+        i = 1;
+      else if (i == 1)
+        i = 0;
+    }
   switch (i)
     {
     case 0:
@@ -694,6 +735,13 @@ gail_button_set_description (AtkAction      *action,
 
   button = GAIL_BUTTON (action);
 
+  if (button->is_down_arrow)
+    {
+      if (i == 0)
+        i = 1;
+      else if (i == 1)
+        i = 0;
+    }
   switch (i)
     {
     case 0:
@@ -1404,7 +1452,7 @@ find_label_child (GtkContainer *container,
                   child = GTK_WIDGET (tmp_list->data);
                   break;
                 }
-              *index--;
+              (*index)--;
 	    }
         }
        /*
@@ -1434,7 +1482,7 @@ find_label_child (GtkContainer *container,
 	              child = widget;
                       break;
                     }
-                  *index--;
+                  (*index)--;
 	        }
 	    }
         }
@@ -1455,12 +1503,10 @@ get_label_from_button (GtkWidget *button,
                        gboolean  allow_many)
 {
   GtkWidget *child;
-  gint n_labels;
 
   if (index > 0 && !allow_many)
     g_warning ("Inconsistent values passed to get_label_from_button");
 
-  n_labels = 0;
   child = gtk_bin_get_child (GTK_BIN (button));
   if (GTK_IS_ALIGNMENT (child))
     child = gtk_bin_get_child (GTK_BIN (child));
@@ -1473,7 +1519,7 @@ get_label_from_button (GtkWidget *button,
   return child;
 }
 
-static void*
+static void
 count_labels (GtkContainer *container,
               gint         *n_labels)
 {
@@ -1485,7 +1531,7 @@ count_labels (GtkContainer *container,
     {
       if (GTK_IS_LABEL (tmp_list->data))
         {
-          *n_labels++;
+          (*n_labels)++;
         }
        /*
         * Label for button which are GtkTreeView column headers are in a 
@@ -1497,7 +1543,7 @@ count_labels (GtkContainer *container,
 
           widget = gtk_bin_get_child (GTK_BIN (tmp_list->data));
           if (GTK_IS_LABEL (widget))
-            *n_labels++;
+            (*n_labels)++;
         }
       else if (GTK_IS_CONTAINER (tmp_list->data))
         {
